/* $Id: badcode.c,v 1.1 2003/06/09 11:13:39 pho Exp pho $ */

#include <sys/types.h>
#include <stdio.h>
#include <string.h>
#include <strings.h>
#include <stdlib.h>
#include <unistd.h>
#include <fcntl.h>
#include <sys/wait.h>
#include <signal.h>
#include <sys/time.h>
#include <sys/resource.h>


#define BUFS 64

char buf [BUFS];
pid_t pid;

void (*sub)();

void
test(void) {
	int i = 2;
	printf("Hello, world (%d)\n", i);
	return;
}

void
alter(void) {	/* Change one byte in the code */
	int i;
	i = random() % BUFS;
	buf[i] = random() & 0xff;
}

void
dump(char *f) {	/* write the file with the altered program code */
	int fd;
	fd = open(f, O_RDWR|O_CREAT, 0664);
	if (fd == -1) {
		perror(f);
		exit(2);
	}
	write(fd, buf, BUFS);
	close(fd);
}

void
generate(void) {
	int i;
	char file[128];

	for (i = 0; i < 512; i++) {
		if (i % 10 == 0)
			bcopy(test, buf, BUFS);
		sprintf(file, "%05d", i);
		alter();
		dump(file);
	}
}

void
handler(int i) {	/* alarm handler */
	if (pid != 0) {
		kill(pid, SIGHUP);
		kill(pid, SIGKILL);
	}
	exit(1);
}

void
child(char *f) {
	int fd;
	fd = open(f, O_RDWR, 0664);
	if (fd == -1) {
		perror(f);
		exit(2);
	}
	read(fd, buf, BUFS);
	close(fd);

	signal(SIGALRM, handler);
	signal(SIGILL,  handler);
	signal(SIGFPE,  handler);
	signal(SIGSEGV, handler);
	signal(SIGBUS,  handler);
	signal(SIGURG,  handler);
	signal(SIGSYS,  handler);
	signal(SIGTRAP, handler);
	alarm(2);
	(*sub)();
	exit(2);
}

void
run(char **argv, char *f) {
	char file[32];
	struct rlimit rlim;
	int n, status;

	strcpy(file, f);

	rlim.rlim_cur = rlim.rlim_max = 0;
	if (setrlimit(RLIMIT_CORE, &rlim) < 0)
		perror("setrlimit failed");

	if (access(file,R_OK) != -1) {
		pid = fork();
		if (pid == -1) {
			perror("fork");
			exit(2);
		}

		if (pid == 0) {
			child(file);
			printf("%s:%d\n", __FILE__, __LINE__);
		} else {
			signal(SIGALRM, handler);
			alarm(3);
			wait(&status);
			alarm(0);
/*			if (WIFSIGNALED(status))
				printf("%s exit status: %d\n", file,
						WTERMSIG(status));*/
		}
		n = atoi(file);
		sprintf(file, "%05d", ++n);
		if (execl(argv[0], argv[0], file, 0) == -1)
			perror("execl");
	}
}


int
main(int argc, char **argv) {

	if (argc != 2) {
		fprintf(stderr, "Usage: %s {-g|<start file>}\n", argv[0]);
		exit(1);
	}

	sub = (void (*)()) buf;
	srandom(getpid());

	if (!strcmp(argv[1], "-g"))
		generate();
	else
		run(argv, argv[1]);
	return 0;
}
