#include <sys/param.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <netdb.h>
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <signal.h>
#include <string.h>

#define NB (400 * 1024 * 1024)

static int cont = 1;

void
handler(int i) {
	cont = 0;
	exit(3);
}

int port, tim;

void
err(const char *text) {
	perror(text);
	exit(1);
}

void
server(void) {
	int tcpsock, msgsock;
	int on, len;
	struct sockaddr_in inetaddr, inetpeer;
	int n, t, buf[256];

	signal(SIGALRM, handler);
	alarm(tim);

	on = 1;
	if ((tcpsock = socket(AF_INET, SOCK_STREAM, 0)) < 0)
		err("socket");

	if (setsockopt(tcpsock,
	    SOL_SOCKET, SO_REUSEADDR, (char *)&on, sizeof(on)) < 0)
		err("setsockopt");

	inetaddr.sin_family = AF_INET;
	inetaddr.sin_addr.s_addr = INADDR_ANY;
	inetaddr.sin_port = htons(port);
	inetaddr.sin_len = sizeof(inetaddr);

	if (bind(tcpsock,
	    (struct sockaddr *)&inetaddr, sizeof (inetaddr)) < 0)
		err("bind");

	if (listen(tcpsock, 5) < 0)
		err("listen");

	len = sizeof(inetpeer);
	if ((msgsock = accept(tcpsock,
	    (struct sockaddr *)&inetpeer, &len)) < 0)
		err("accept");

	t = 0;
	while (cont) {
		if ((n = read(msgsock, buf, sizeof(buf))) < 0)
			err("read");
		t += n;
		if (n == 0) break;
	}
	return;
}

void
client(void) {
	int tcpsock, on;
	struct sockaddr_in inetaddr;
	struct hostent *hostent;
	int i, buf[256];

	signal(SIGALRM, handler);
	alarm(tim);

	on = 1;
	if ((tcpsock = socket(AF_INET, SOCK_STREAM, 0)) < 0)
		err("socket");

	if (setsockopt(tcpsock,
	    SOL_SOCKET, SO_REUSEADDR, (char *)&on, sizeof(on)) < 0)
		err("setsockopt");

	hostent = gethostbyname ("localhost");
	memcpy (&inetaddr.sin_addr.s_addr, hostent->h_addr,
		sizeof (struct in_addr));

	inetaddr.sin_family = AF_INET;
	inetaddr.sin_addr.s_addr = INADDR_ANY;
	inetaddr.sin_port = htons(port);
	inetaddr.sin_len = sizeof(inetaddr);

	if (connect (tcpsock, (struct sockaddr *) &inetaddr,
		sizeof(inetaddr)) < 0)
		err("connect");
	for (i = 0; i < sizeof(buf)/sizeof(int); i++)
		buf[i] = i;

	while (cont)
		for (i = 0; i < NB; i+= sizeof(buf)) {
			write(tcpsock, buf, sizeof(buf));
			if (cont == 1) break;
		}

	close(tcpsock);
	return;
}

int
main(int argc, char **argv) {
	pid_t pid;

	if (argc != 3) {
		fprintf(stderr, "Usage: %s <time> <port>\n", argv[0]);
		return 1;
	}


	sleep(1);	/* Wait for siblings to start */
	tim = atoi(argv[1]);
	sscanf(argv[2], "%d", &port);

	if ((pid = fork()) == 0)
		client();
	else if (pid > 0)
		server();
	else 
		err("fork");

	return 0;
}
