#include <sys/syscall.h>
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <signal.h>
#include <sys/wait.h>
#include <time.h>

pid_t pid;

long calls = 0;
long kills = 0;

void
handler(int i) {
	if (pid == 0)
		printf("%ld calls, %ld kills\n", calls, kills);
	exit(0);
}

void
inthandler(int i) {
	signal(SIGINT, inthandler);
	kills++;
	return;
}

int main(int argc, char **argv) {
	long arg1, arg2, arg3, arg4, arg5;
	int sec=2, num, status;
	time_t then, now;

	if (argc != 3) {
		fprintf(stderr, "Usage: %s <syscall nb>  <secs>\n", argv[0]);
		return 1;
	}

	sscanf(argv[1], "%d", &num);
	sscanf(argv[2], "%d", &sec);

	srandom(getpid());
	then = time(&now);

	pid = fork();
	if (pid == -1) {
		perror("fork");
		exit(1);
	}

	if (pid == 0) {
		signal(SIGBUS, SIG_IGN);
		signal(SIGSEGV, SIG_IGN);
		signal(SIGSYS, SIG_IGN);
		signal(SIGINT, inthandler);
		signal(SIGALRM, handler);
		alarm(sec);
		printf("Testing syscall #%d\n", num);
		while(1) {
			arg1 = random();
			arg2 = random();
			arg3 = random();
			arg4 = random();
			arg5 = random();
			if (calls++ % 100)
				if (time(&now) - then > sec)
					break;
			syscall(num, arg1, arg2, arg3, arg4, arg5);
		}
		printf("%ld calls, %ld kills\n", calls, kills);
	} else {
		usleep(5000);	/* 0.005 sec */
		 while (kill(pid, SIGINT) == 0)
			if (calls++ % 100) {
				if (time(&now) - then > sec)
					break;
			}
		(void)kill(pid, SIGKILL);
		wait(&status);
	}
	return 0;
}
